"""
The main CLI for pftf.
"""

# Copyright 2008, Markus Demleitner.  This file is under the GPL,
# see COPYING for details.

from datetime import date, timedelta
import os
import sys
import time

from pftf import due
from pftf import makecli
from pftf import settings
from pftf import timefile
from pftf import times


class Error(settings.Error):
	pass


def formatDist(dist):
	toDisp = [(int(v*100), k) for k, v in dist.iteritems()]
	toDisp.sort()
	return "\n".join(["%20s -- %2d%%"%(label, p) for p, label in toDisp])


def printBasicReport(opts, times):
	print "Total Pining  -- %.2f hours"%times.getWorkTime()
	if opts.showDist:
		print formatDist(times.getWorkTimeDist())


def printForDay(opts):
	printBasicReport(opts, times.Times.forDateStart(
		date(opts.year, opts.month, opts.day).strftime(
			settings.get("dateformat"))))


def reportOnInterval(opts, workTimes, hoursDue):
	printBasicReport(opts, workTimes)
	stillLeft = hoursDue-workTimes.getWorkTime()
	if opts.stopToday:
		if stillLeft>0:
			print "You are %.2f hours behind current due"%stillLeft
		else:
			print "You are %.2f hours ahead of current due"%abs(stillLeft)
	else:
		print "Hours still due: %.2f hours (%.2f mean days)"%(stillLeft,
			stillLeft/settings.get("hoursPerDay"))


def reportOnMonth(opts):
	hoursDue = due.getHoursForMonth(opts.month, opts.year, opts.stopToday)
	reportOnInterval(opts, times.Times.forMonth(opts), hoursDue)


def getTimesForWeek(opts):
	return times.Times.forDates(set(d.strftime(settings.get("dateformat"))
		for d in due.iterDaysForWeek(opts.day, opts.month, opts.year)),
		withArchive=opts.withArchive)


def reportOnWeek(opts):
	tForWeek = 	getTimesForWeek(opts)
	hoursDue = due.getHoursForWeek(opts.day, opts.month, opts.year, 
		opts.stopToday)
	reportOnInterval(opts, tForWeek, hoursDue)


def edit(opts):
	"""brings up $VISUAL with the pining file.
	"""
	if not "VISUAL" in os.environ:
		raise Error("You need to specify your editor in the VISUAL env var")
	tf = timefile.TimeFile()
	tf.lock()
	try:
		os.system("%s '%s'"%(os.environ["VISUAL"], tf.fName))
	finally:
		tf.unlock()


def archive(opts):
	"""stuff everything not for the current month into the archive file.
	"""
	newCurrentLines = []
	curFile = timefile.TimeFile()
	archFile = timefile.ArchiveFile()
	try:
		fIn = curFile.getReadingFile()
		fOut = archFile.getAppendingFile()
		startForCur = date(opts.year, opts.month, opts.day
			).strftime(settings.get("monthformat"))
		for ln in fIn:
			if ln.startswith(startForCur):
				newCurrentLines.append(ln)
			else:
				fOut.write(ln)
		fIn.close()
		fOut.close()
		fOut = curFile.getEmptiedFile()
		fOut.write("".join(newCurrentLines))
		fOut.close()
	finally:
		curFile.unlock()
		archFile.unlock()


def putStamp(opts, label=None):
	if label is None:
		timefile.TimeFile().append(timefile.makeStamp())
	else:
		timefile.TimeFile().append("%s\t%s"%(timefile.makeStamp(), label))


actions = {
	"day": printForDay,
	"month": reportOnMonth,
	"week": reportOnWeek,
	"archive": archive,
	"edit": edit,
	"makecli": makecli.main,
	"quit": putStamp,
}

graphAvailable = False
try:
	from pftf import graph
	actions["graph"] = graph.makeMonthGraph
	graphAvailable = True
except ImportError:  # no pychart, no graphics
	pass


def printConfigDocs():
	"""outputs the configuration documentation to stdout.
	"""
# Tricky stuff: We import settings as __main__, which makes it output
# its docs.  This way, we avoid loading user configuration.
	import pftf, imp
	sef = imp.find_module("settings", pftf.__path__)
	imp.load_module("__main__", *sef)


def printHolidays(opts):
	thisYear = date.today().year
	h = due.Holidays(thisYear)
	fixedNames = set([l for l, _ in h.fixedHolidays])
	easterNames = set([l for l, _ in h.easterBoundHolidays])
	for d, name in h:
		print "* %s"%name
		if name in easterNames:
			print "   Easter bound, in %d: %s"%(thisYear, d)
		else:
			print "   Date bound, %s"%d
		if name in due.alternativeNames:
			print "   Also known as %s"%(", ".join(due.alternativeNames[name]
				).encode("utf-8"))


def formatErrMsg(exception):
	if isinstance(exception, settings.Error):
		sys.stderr.write("%s: %s\n"%(sys.argv[0], 
			unicode(exception).encode("utf-8")))
	else:
		raise
	sys.exit(1)


def parseCommandLine():
	from optparse import OptionParser
	parser = OptionParser(usage="%%prog [options] [<action>]"
		" -- compute pining times.\n Actions include: %s\n"
		"    you can also pass a wClass as action."%", ".join(actions))

	today = date.today()
	parser.add_option("-a", "--with-archive", help="add archived data"
		" for computation", dest="withArchive", action="store_true", 
		default=False)
	parser.add_option("-d", "--day", help="output for DAY instead of current"
		" day", dest="day", action="store", type="int", metavar="DAY",
		default=today.day)
	parser.add_option("-m", "--month", help="output for MONTH instead of current"
		" month", dest="month", action="store", type="int", metavar="MONTH",
		default=today.month)
	parser.add_option("-y", "--year", help="output for YEAR instead of current"
		" year", dest="year", action="store", type="int", metavar="YEAR",
		default=today.year)
	parser.add_option("-r", "--relative", help="compare work done and"
		" work due until today of month and week actions", action="store_true",
		dest="stopToday", default=False)
	parser.add_option("-o", "--output-to", help="output graphics to FILE",
		dest="outputName", action="store", type="str", metavar="FILE",
		default=settings.get("plot", "defaultOutputName"))
	parser.add_option("-p", "--percents", help="output distribution"
		" by label", dest="showDist", action="store_true")
	parser.add_option("-H", "--help-config", help="Show help on items"
		" available for %s/config and exit"%settings.dataDir,
		dest="showConfig", action="store_true", default=False)
	parser.add_option("-S", "--help-holidays", help="Show holidays"
		" known to pftf and exit",
		dest="showHolidays", action="store_true", default=False)
	if graphAvailable:
		parser.add_option("-C", "--help-colours", help="Show available"
			" colours and exit", action="store_true", default=False,
			dest="showColours")

	opts, args = parser.parse_args()

	if opts.showConfig:
		printConfigDocs()
		sys.exit(0)
	if graphAvailable and opts.showColours:
		graph.printColors(opts)
		sys.exit(0)
	if opts.showHolidays:
		printHolidays(opts)
		sys.exit(0)

	if len(args)>1:
		parser.print_help()
		sys.exit(1)
	elif len(args)==1:
		action = args[0]
	else:
		action = "day"
	if not action in actions and not action in settings.get("wClasses"):
		parser.print_help()
		sys.exit(1)

	# relative dates are a complete mess.  Sorry 'bout this, but I can't
	# see how to get reasonable semantics in a simpler way.
	if opts.day<0:
		today = today+timedelta(opts.day)
		if opts.year>0:
			opts.year = today.year
		if opts.month>0:
			opts.month = today.month
	if opts.year<0:
		opts.year += today.year
	if opts.year<100:
		opts.year+=2000
	opts.day = today.day
	if opts.month<=0:
		opts.month += today.month
		while opts.month<=0:
			opts.year -= 1
			opts.month += 12
	return opts, action


def main():
	try:
		opts, action = parseCommandLine()
		if action in actions:
			actions[action](opts)
		else:
			putStamp(opts, action)
	except Exception, ex:
		formatErrMsg(ex)
