# -*- coding: iso-8859-1 -*-
"""
Computation of the work due, including holiday calculation.
"""

# Copyright 2008, Markus Demleitner.  This file is under the GPL,
# see COPYING for details.

import math
from datetime import date, timedelta

from pftf import settings

alternativeNames = {
	"NewYear": ["Neujahrstag"],
	"Epiphany": [u"Hl. Drei K\xf6nige"],
	"GoodFriday": ["Karfreitag"],
	"EasterMonday": ["Ostermontag"],
	"RealLabourDay": ["Tag der Arbeit"],
	"AscensionChrist": ["Christi Himmelfahrt"],
	"CorpusChristi": ["Fronleichnam"],
	"PentecostMonday": ["Pfingstmontag"],
	"AssumptionMary": [u"Mari\xe4 Himmelfahrt"],
	"LumpingFeast": ["Tag der Deutschen Einheit"],
	"ReformationDay": ["Reformationstag"],
	"AllSaints": ["Allerheiligen"],
	"ChrisEve": ["Heiliger Abend"],
	"Christmas1": ["1. Weihnachtstag"],
	"Christmas2": ["2. Weihnachtstag"],
	"NYEve": ["Sylvester"],
}


class Holidays(object):
	"""is a container for the computation and representation of holidays
	of a particular year.
	"""
	def __init__(self, year):
		self.feasts, self.dates = {}, {}
		self._computeFixed(year)
		self._computeEasterBound(year)

	def __iter__(self):
		dates = self.dates.keys()
		dates.sort()
		for d in dates:
			yield d, self.dates[d]

	def __contains__(self, d):
		return d in self.dates

	def isPertaining(self, d):
		return d in self and self.dates[d] in settings.get("holidays")

	fixedHolidays = [ 
		("NewYear", (1,1)),
		("Epiphany", (6,1)),
		("RealLabourDay", (1,5)),
		("AssumptionMary", (15,8)),
		("LumpingFeast", (3,10)),
		("ReformationDay", (31,10)),
		("AllSaints", (1,11)),
		("ChrisEve", (24,12)),
		("Christmas1", (25,12)),
		("Christmas2", (26,12)),
		("NYEve", (31,12)),
	]

	def _computeFixed(self, year):
		for feast, (day, month) in self.fixedHolidays:
			d = date(year, month, day)
			self.dates[d] = feast
			self.feasts[feast] = d

	easterBoundHolidays = [
		("GoodFriday", -2),
		("EasterMonday", +1),
		("AscensionChrist", +40),
		("PentecostMonday", +50),
		("CorpusChristi", +60),
	]

	def _computeEaster(self, year):
		G = year%19
		C = year/100
		H = (C-C/4-(8*C+13)/25+19*G+15)%30
		I = H-H/28*(1-29/(H+1))*((21-G)/11)
		J = (year+year/4+I+2-C+C/4)%7
		L = I-J
		if L<4:
			month, day = 3, L+28 	
		else:
			month, day = 4, L-3
		return date(year, month, day)

	def _computeEasterBound(self, year):
		easter = self._computeEaster(year)
		for feast, dayDiff in self.easterBoundHolidays:
			d = easter+timedelta(dayDiff)
			self.dates[d] = feast
			self.feasts[feast] = d
	

def memoized(fun):
	cache = {}
	def funWithMemo(*args):
		if args not in cache:
			cache[args] = fun(*args)
		return cache[args]
	return funWithMemo


@memoized
def getHolidaysFor(year):
	return Holidays(year)


def iterDaysForMonth(month, year):
	for day in range(1, 32):
		try:
			yield date(year, month, day)
		except ValueError: # month shorter than 31 days
			break


def getStartOfWeek(day, month, year):
	"""returns a date instance for the beginning of the week day.month.year
	is in.
	"""
	cur = date(year, month, day)
	return cur-timedelta(cur.weekday())


def iterDaysForWeek(day, month, year):
	day = getStartOfWeek(day, month, year)
	for weekday in range(7):
		yield day+timedelta(weekday)


def iterDaysBetween(date1, date2):
	"""iterates over all days between date1 and date2 *inclusive*.
	"""
	oneDay = timedelta(1)
	curDate = date1
	while curDate<=date2:
		yield curDate
		curDate = curDate+oneDay


def filterWorkdays(dateIter, stopToday=False):
	"""returns only those days from the date-yielding dateIter that work
	is required on.
	"""
	endDate = None
	if stopToday:
		endDate = date.today()
	for d in dateIter:
		if (d.weekday() in settings.get("workDays") and 
				not getHolidaysFor(d.year).isPertaining(d)):
			yield d
		if endDate and endDate<=d:
			break


def iterWorkdaysBetween(date1, date2):
	"""iterates over the work days between date1 and date2 inclusive.
	"""
	return filterWorkdays(iterDaysBetween(date1, date2))


def getHoursForMonth(month, year, stopToday=False):
	"""returns the number of work hours in month of year.
	"""
	return sum(settings.get("hoursPerDay") for wd in filterWorkdays(
		iterDaysForMonth(month, year), stopToday))


def getHoursForWeek(day, month, year, stopToday=False):
	"""returns the number of work hours in the week day.month.year is in.
	"""
	return sum(settings.get("hoursPerDay") for wd in filterWorkdays(
		iterDaysForWeek(day, month, year), stopToday))


if __name__=="__main__":
	print getHoursForWeek(8, 10, 2008)
