"""
Configuration definitions for pftf.
"""

# Copyright 2008, Markus Demleitner.  This file is under the GPL,
# see COPYING for details.

import os
import re

from pftf import fancyconfig
from pftf.fancyconfig import Configuration, Section, ListConfigItem,\
	DefaultSection, SetConfigItem, StringConfigItem, ParseError,\
	IntSetConfigItem, IntConfigItem, MagicSection, FloatConfigItem,\
	BytestringConfigItem, EnumeratedConfigItem, BooleanConfigItem,\
	ConfigError


class Error(Exception):
	"""is the base class for (almost) all excptions raised by pftf.
	"""
	pass


dataDir = os.environ.get("PFTFDATA",
	os.path.join(os.environ["HOME"], ".pftf"))
if not os.path.exists(dataDir):
	os.makedirs(dataDir, 0700)


class FontConfigItem(StringConfigItem):
	"""is a config item for a simple tkinter font specifier.
	"""
	def _parse(self, value):
		raw = StringConfigItem._parse(self, value)
		try:
			mat = re.match("(.*),(\d+)\s*$", raw)
			face, size = mat.groups()
			size = int(size)
		except (AttributeError, ValueError):
			raise ParseError("'%s' is not a syntactically valid font specifier."
				" Try Helvetica,8 or something like this."%value.encode("utf-8"))
		return face, size
	
	def _unparse(self, value):
		return "%s,%d"%value


_config = Configuration(
	DefaultSection("Setting work classes, holidays, and such",
		ListConfigItem("wClasses", "Vac,Sic,Tra,Dev,Oth",
			description="List of the labels available for work time.  Be short,"
				" these labels take up screen space in the timekeeping GUI"),
		SetConfigItem("dayClasses", "Vac, Sic, Tra",
			description="Elements of wClasses that should not be shown in the"
				" timekeeping GUI (because the activities usually take an entire"
				" day and are handled trough pineday"),
		IntSetConfigItem("workDays", "0, 1, 2, 3, 4",
			description="Indices of weekdays you are supposed to work on"
				" regularly, Monday=0.  This is only used to compute your hours"
				" due."),
		SetConfigItem("holidays", "NewYear, Epiphany, GoodFriday,"
			"EasterMonday, RealLabourDay, AscensionChrist, CorpusChristi, "
			"PentecostMonday, LumpingFeast, AllSaints, Christmas1, Christmas2",
			description="Public holidays in effect in your jurisdiction, used"
				" for the computation of the work due.  To see the available"
				" holidays, say 'pining holidays'.  You may want to retrofit"
				" your holidays in pftf.due.  Please feed back your patches."),
		FloatConfigItem("hoursPerDay", 8, description="Daily work time"
			" (usually computed as (weekly work time)/5 or so), in hours"),
		BytestringConfigItem("dateformat", "%Y-%m-%d", description=
			"Date format used.  Do not change."),
		BytestringConfigItem("monthformat", "%Y-%m", description=
			"Part of the date format covering year and month.  Do not change."),
		BytestringConfigItem("isoformat", "%Y-%m-%dT%H:%M:%S",
			description="ISO time format.  Do not change."),
		BooleanConfigItem("allowBlanksInLabels", "False", description=
			"Allow blanks in labels.  If you enable this, you must never"
			" edit the WTF with an editor that converts tabs"),
	),
	Section("filenames", "File names (the base directory is in PFTFPATH)",
		StringConfigItem("piningFile", "pining.tsv",
			description="Name of the file containing current time data."),
		StringConfigItem("archiveFile", "pined.tsv",
			description="Name of the file containing archived time data."),
	),
	Section("gui", "Settings related to the timekeeping GUI; colours in"
			"here are Tk colours, i.e., #rrggbb works.",
		FontConfigItem("labelFont", 'Helvetica,8', "Font to use for the work"
			" labels as face,size.  Basically, we understand what tk understands."),
		StringConfigItem("highlight", "#ff0000", "Colour of labels that are"
			" selected"),
		StringConfigItem("unselected", "black", "Colour of labels that are"
			" not selected"),
		StringConfigItem("background", "#a0ff8d", "Background colour"),
		IntConfigItem("width", "50", "Width of the GUI window (the height"
			" will be computed from wClasses)."),
		IntConfigItem("hSpace", "5", "Pixels of horizontal space between two"
			" labels."),
		StringConfigItem("title", "pftf", "Window title"),
		IntConfigItem("pollInterval", "1000", "Interval UI should check for"
			" changes on WTF in milliseconds, 0 to turn off."),
	),
	Section("plot", "Settings related to work time plots",
		IntConfigItem("width", "600", "Width of the month plot, in point"),
		IntConfigItem("height", "400", "Height of the month plot, in point"),
		IntConfigItem("pieRadius", "100", "Radius of pie plots, in point"),
		EnumeratedConfigItem("format", "pdf", "Format of the generated"
			" output file", options=["ps", "pdf", "svg", "x11", "png"]),
		StringConfigItem("defaultOutputName", "pined.pdf", "Name of"
			" the graphics output, - for stdout"),
	),
	MagicSection("classCols", "Assignment of colours to labels in graphs."
		"  Note that the colours here are from the X11 colour database (rgb.txt)."
		"  To see what's available, say 'pining colours'.",
		itemFactory=StringConfigItem, defaults=(("Vac", "white"),
			("Sic", "gray30"), ("Tra", "skyblue"), ("Dev", "springgreen"),
			("Oth", "salmon"))
	),
)


get = _config.get
set = _config.set



if __name__=="__main__":
	print fancyconfig.makeTxtDocs(_config)
else:
	try:
		fancyconfig.readConfiguration(_config, None, 
			os.path.join(dataDir, "config"))
	except ConfigError, msg:
		import sys
		sys.stderr.write("%s: %s\n"%(sys.argv[0], unicode(msg).encode("utf-8")))
		sys.exit(1)
