#!/usr/bin/env python
"""
A dockapp-like thing to acquire times and dates you worked and what you worked
on.
"""

# Copyright 2008, Markus Demleitner.  This file is under the GPL,
# see COPYING for details.

from datetime import datetime
import time
import Tkinter
import tkMessageBox

from pftf import settings
from pftf import timefile


class TimeTracker(Tkinter.Tk):
	breakText, unbreakText = "Break", "Unbreak"

	def __init__(self, wClasses):
		self.wClasses = wClasses
		Tkinter.Tk.__init__(self)
		self.config(background=settings.get("gui", "background"))
		self.title(settings.get("gui", "title"))
		self.timefile = timefile.TimeFile()
		self.inBreak = False
		self._makeWidgets()
		self._makeItems()
		self._layout()
		self._resize()
		self._lastStamp = self.timefile.getTimestamp()
		self.resizable(False, False)
		self.canvas.tag_bind("inactive", '<1>', self._activate)
		self.canvas.tag_bind("active", '<1>', self._deactivateAll)
		self.bind('q', lambda ev: self.quit())
		self.pollInterval = settings.get("gui", "pollInterval")
		if self.pollInterval:
			self.after(self.pollInterval, self._pollWTFChange)

	def _makeWidgets(self):
		self.breakButton = Tkinter.Button(self, text=self.breakText,
			font=settings.get("gui", "labelFont"), padx=0, pady=0, 
			relief=Tkinter.RAISED,
			command=self._doBreak, background=settings.get("gui", "background"),
			activebackground=settings.get("gui", "background"), bd=2, 
			highlightthickness=0)
		self.breakButton.grid(row=0, column=1, sticky=Tkinter.E+Tkinter.W)
		self.canvas = Tkinter.Canvas(self, 	bd=0, highlightthickness=0,
			background=settings.get("gui", "background"))
		self.canvas.grid(row=1, column=1, sticky=Tkinter.NE+Tkinter.SW)
		self.columnconfigure(1, weight=1)
		self.rowconfigure(1, weight=1)

	def _makeItem(self, wClass):
		curItem = self.canvas.create_text(0, 0, text=wClass, 
			anchor=Tkinter.NW, font=settings.get("gui", "labelFont"), 
			tags=("inactive",))
		self.items[curItem] = wClass
		self.tagMap[wClass] = curItem
		x0, y0, x1, y1 = self.canvas.bbox(curItem)
		tw, th = x1-x0, y1-y0
		self.itemProperties.append((wClass, tw, th))

	def _makeItems(self):
		self.items, self.tagMap, self.itemProperties = {}, {}, []
		for tag in self.wClasses:
			self._makeItem(tag)

	def _updateFromTimeFile(self):
		try:
			event, payload = timefile.lexLine(self.timefile.getLastLine())
			if event==timefile.ENDTIME:
				label = None
			elif event==timefile.STARTTIME:
				label = payload[1]
			else:
				raise ValueError("Nothing to do for me")
			if label is None:
				self._resetAll()
			else:
				if label in self.tagMap:
					self._resetAll()
					self._activateItem(self.tagMap[label])
				else:
					raise ValueError("Invalid label")
		except ValueError:  # don't bomb out on this
			pass
		self._lastStamp = self.timefile.getTimestamp()

	def _pollWTFChange(self):
		try:
			curTS = self.timefile.getTimestamp()
			if curTS!=self._lastStamp:
				self._updateFromTimeFile()
		finally:
			self.after(self.pollInterval, self._pollWTFChange)

	def _resize(self):
		estHeight = self.breakButton.winfo_reqheight()
		_, y0, _, y1 = self.canvas.bbox(Tkinter.ALL)
		self.geometry("%dx%d"%(settings.get("gui", "width"), estHeight+y1-y0))

	def _layout(self):
		canvasWidth = settings.get("gui", "width")
		if canvasWidth<10 or not self.itemProperties:
			return

		def collectALine(firstItem, items):
			itemsInLine = [firstItem]
			curWidth, curHeight = firstItem[1], firstItem[2]
			try:
				while True:
					item = items.next()
					_, width, height = item
					if curWidth+settings.get("gui", "hSpace")+width>canvasWidth:
						return item, itemsInLine, curHeight
					else:
						itemsInLine.append(item)
						curHeight = max(curHeight, height)
						curWidth += settings.get("gui", "hSpace")+width
			except StopIteration:
				pass
			return None, itemsInLine, curHeight

		def layoutLine(itemsInLine, cury):
			curx = 0
			for label, width, _ in itemsInLine:
				self.canvas.coords(self.tagMap[label], curx, cury)
				curx += width+settings.get("gui", "hSpace")

		cury = 0
		items = iter(self.itemProperties)
		lastItem = items.next()
		while lastItem:
			lastItem, itemsInLine, lineHeight = collectALine(lastItem, items)
			layoutLine(itemsInLine, cury)
			cury += lineHeight
		self.canvas.config(scrollregion=self.canvas.bbox(Tkinter.ALL))

	def _doBreak(self):
		if self.inBreak:
			self.breakButton.configure(text=self.breakText, relief=Tkinter.RAISED,
				background=settings.get("gui", "background"), 
				activebackground=settings.get("gui", "background"))
			self.inBreak = False
			self.stamp()
		else:
			self.breakButton.configure(text=self.unbreakText, relief=Tkinter.SUNKEN,
				background=settings.get("gui", "highlight"), 
				activebackground=settings.get("gui", "highlight"))
			self.stamp(False)
			self.inBreak = True

	def _resetAll(self):
		self.canvas.itemconfig(Tkinter.ALL, tags=("inactive",), 
			fill=settings.get("gui", "unselected"))

	def _activate(self, ev):
		self._resetAll()
# Work around "issue" where Tkinter.CURRENT is empty when mouse cursor
# hasn't been entering and leaving the item
		item = self.canvas.find_withtag(Tkinter.CURRENT)
		if not item:
			item = self.canvas.find_closest(self.canvas.canvasx(ev.x), 
				self.canvas.canvasy(ev.y))
		self._activateItem(item)
		self.stamp()
	
	def _activateItem(self, item):
		self.canvas.itemconfig(item, tags=("active",), 
			fill=settings.get("gui", "highlight"))

	def _deactivateAll(self, ev):
		self._resetAll()
		self.stamp()

	def getActive(self):
		selected = self.canvas.find_withtag("active")
		if selected:
			return selected[0]

	def stamp(self, overrideSelection=None):
		if self.inBreak:
			return
		self.config(cursor="watch")
		self.update_idletasks()
		try:
			if overrideSelection is None:
				selected = self.getActive()
			else:
				selected = overrideSelection
			try:
				if selected:
					self.timefile.append("%s\t%s"%(timefile.makeStamp(),
						self.items[selected]), timeout=1)
				else:
					self.timefile.append(timefile.makeStamp(), timeout=1)
				self._lastStamp = self.timefile.getTimestamp()
			except timefile.TimeoutReached, msg:
				tkMessageBox.showerror(title="Stale lock?",
					message="%s\nEvent has NOT been recorded."%str(msg))
				raise
		finally:
			self.config(cursor="left_ptr")


def main():
	tt = TimeTracker([c for c in settings.get("wClasses")
		if c not in settings.get("dayClasses")])
	try:
		tt.mainloop()
	finally:
		timefile.TimeFile().append(timefile.makeStamp())


if __name__=="__main__":
	main()
