"""
Tests for reading and writing time files.
"""

import datetime
import os
import shutil
import time
import threading
import unittest

from pftf import settings
from pftf import timefile

class ParserTest(unittest.TestCase):
	"""tests for correct operation of the file parser.
	"""
	def _getIntervalsFor(self, lineList):
		p = timefile.LineListParser()
		p.feedList(lineList)
		return p.intervals

	def testSimpleInterval(self):
		ints = self._getIntervalsFor(["2008-10-01T10:01:49	Qual",
			"2008-10-01T10:16:36"])
		start, duration, label = ints[0]
		self.assertEqual(start, datetime.datetime(2008, 10, 1, 10, 1, 49))
		self.assertEqual(duration, 887)
		self.assertEqual(label, 'Qual')
	
	def testIntervalWithDay(self):
		"""tests for intervening day items not disturbing intervals.
		"""
		ints = self._getIntervalsFor(["2008-10-01T10:01:49\tQual",
			"2008-09-12\tVac",
			"2008-10-01T10:16:36"])
		start, duration, label = ints.pop()
		self.assertEqual(label, "Qual")
		start, duration, label = ints.pop()
		self.assertEqual(label, "Vac")
		self.assertEqual(duration, settings.get("hoursPerDay")*3600)
		self.assertEqual(start, datetime.datetime(2008, 9, 12, 0, 0))
		self.assertEqual(ints, [])

	def testOpenInterval(self):
		"""tests for closing of intervals open at the end.
		"""
		start = "%s\tQual"%time.strftime(
			settings.get("isoformat"), time.localtime())
		time.sleep(1)
		intv = self._getIntervalsFor([start])
		self.assertEqual(len(intv), 1, "Open intervals are not closed")
		self.assertEqual(intv[0][2], "Qual")
		self.assert_(intv[0][1]<200)

	def testMixedIntervals(self):
		ints = self._getIntervalsFor(["2008-10-01T10:01:49\tQual",
			"2008-10-01T10:16:36\tDev",
			"2008-10-01T10:16:38",
			"2008-10-01T10:16:40",
			"2008-10-01T10:16:42\tDev",
			"2008-10-01T10:26:40",])
		self.assertEqual(len(ints), 3)
		self.assertEqual([i[2] for i in ints], ["Qual", "Dev", "Dev"])
		self.assertEqual([i[1] for i in ints], [887, 2, 598])

	def testDaysWithTimes(self):
		ints = self._getIntervalsFor(["2008-10-02\tTra\t10",
			"2008-10-03\tTra\t3"])
		self.assertEqual(len(ints), 2)
		self.assertEqual([i[1] for i in ints], [36000.0, 10800.0])

	def testRaising(self):
		# no closer for old timestamps
		self.assertRaises(timefile.ParseError, self._getIntervalsFor, 
			['2000-01-01T00:00:00\tDev'])
		# Too many fields
		self.assertRaises(timefile.ParseError, self._getIntervalsFor,
			['2000-01-01T00:00:00\tDev\tx'])
		# Bad time spec
		self.assertRaises(timefile.ParseError, self._getIntervalsFor,
			['yodl-01-01X\tBla'])


class TimeFileTest(unittest.TestCase):
	"""tests aspects of TimeFile access.
	"""
	def setUp(self):
		self.origDir = settings.dataDir
		settings.dataDir = "testData"
		if os.path.isdir(settings.dataDir):
			shutil.rmtree(settings.dataDir)
		os.makedirs(settings.dataDir)
		timefile.TimeFile().append("# first line")

	def tearDown(self):
		shutil.rmtree(settings.dataDir)
		settings.dataDir = self.origDir

	def testBasic(self):
		t = timefile.TimeFile()
		t.append("line1")
		t.append("line2")
		self.assertEqual(t.read(), "# first line\nline1\nline2\n")
	
	def testLocking(self):

		def accessFile(eventOpen, eventFree):
			t = timefile.TimeFile()
			f = t.getAppendingFile()
			eventOpen.set()
			eventFree.wait()
			f.close()

		# start first thread
		ev1Free, ev1Open = threading.Event(), threading.Event()
		t1 = threading.Thread(target=accessFile, args=(ev1Open, ev1Free))
		t1.setDaemon(True)
		t1.start()
		# wait for thread to open the file
		ev1Open.wait(0.2)
		self.assert_(ev1Open.isSet(), "First open of file failed")
		# start second thread
		ev2Free, ev2Open = threading.Event(), threading.Event()
		t2 = threading.Thread(target=accessFile, args=(ev2Open, ev2Free))
		t2.setDaemon(True)
		t2.start()
		# t2 must not be able to open the file
		ev2Open.wait(0.2)
		self.assert_(not ev2Open.isSet(), "Second open of file succeeded"
			" despite lock")
		# allow t1 to close the file
		ev1Free.set()
		# t2 must now have opened the file
		ev2Open.wait(0.2)
		self.assert_(ev2Open.isSet(), "Second open of file didn't happen"
			" on lock release")
		ev2Free.set()
		t1.join(1)
		t2.join(1)

	def testLastLine(self):
		"""tests for provision of the WTF's last line access.
		"""
		t = timefile.TimeFile()
		t.append("line1")
		self.assertEqual(t.getLastLine(), "line1")
		t.append("line2")
		self.assertEqual(t.getLastLine(), "line2")
		f = t.getEmptiedFile()
		f.close()
		self.assertEqual(t.getLastLine(), None)


if __name__=="__main__":
	import testhelpers
	testhelpers.main(TimeFileTest)
