"""
Tests for the Times class and the due module.
"""

import datetime
import os
import shutil
import unittest
from datetime import date

import testhelpers

from pftf import due
from pftf import settings
from pftf import timefile
from pftf import times



testData = """2008-10-01T08:45:00	Admin
2008-10-01T09:45:00	Qual
2008-10-01T11:45:00
2008-10-01T12:00:00	Dev
2008-10-01T16:00:00	Mail
2008-10-01T17:00:00
2008-10-02T10:07:24	Mail
2008-10-02T10:15:00	Admin
2008-10-02T10:30:00	Qual
2008-10-02T12:15:36
2008-10-02T12:45:00	Dev
2008-10-02T17:00:00
2008-10-07	Vac
2008-11-05T10:46:00	Mail
2008-11-05T10:50:00	Qual
2008-11-05T12:15:36
2008-11-05T12:45:00	Dev
2008-11-05T17:00:00
"""


class WithTestDataTest(testhelpers.CleanSlateTest):
	"""is a base class for tests that want testData in pining.txt.
	"""
	def setUp(self):
		testhelpers.CleanSlateTest.setUp(self)
		f = timefile.TimeFile().getAppendingFile()
		f.write(testData)
		f.close()


class TestTimesConstruction(WithTestDataTest):
	"""tests for construction of Times instances from Times files.
	"""
	def testForDay(self):
		t = times.Times.forDateStart("2008-10-01")
		self.assertEqual(t.getWorkTime(), 8)
		self.assertEqual(t.getWorkTimeDist(), {'Admin': 0.125, 'Qual': 0.25, 
			'Dev': 0.5, 'Mail': 0.125})
		days = [d for d in t.iterdays()]
		self.assertEqual(days, [datetime.date(2008, 10, 1)])

	def testForDates(self):
		t = times.Times.forDates(set(["2008-10-01", "2008-10-03",
			"2008-10-07"]))
		self.assertEqual(t.getWorkTime(), 16)
		self.assertEqual(t.getWorkTimeDist(), {'Admin': 0.0625, 'Qual': 0.125, 
			'Dev': 0.25, 'Mail': 0.0625, 'Vac': 0.5})
		days = [d for d in t.iterdays()]
		self.assertEqual(days, [datetime.date(2008, 10, 1),
			datetime.date(2008, 10, 7)])


class HolidayTest(unittest.TestCase):
	"""tests for due's holiday class
	"""
	def setUp(self):
		self.holidays = due.getHolidaysFor(2007)
		self.expected = {'Christmas2': date(2007, 12, 26), 
			'NewYear': date(2007, 1, 1), 
			'AllSaints': date(2007, 11, 1), 'Christmas1': date(2007, 12, 25), 
			'RealLabourDay': date(2007, 5, 1), 'AscensionChrist': date(2007, 5, 18), 
			'GoodFriday': date(2007, 4, 6), 'ChrisEve': date(2007, 12, 24), 
			'CorpusChristi': date(2007, 6, 7), 'Epiphany': date(2007, 1, 6), 
			'AssumptionMary': date(2007, 8, 15), 
			'ReformationDay': date(2007, 10, 31), 
			'NYEve': date(2007, 12, 31), 'PentecostMonday': date(2007, 5, 28), 
			'EasterMonday': date(2007, 4, 9), 'LumpingFeast': date(2007, 10, 3)}

	def testLabelsPresent(self):
		self.assertEqual(set(self.expected), set(self.holidays.feasts))
	
	def testDates(self):
		for label, day in self.expected.iteritems():
			self.assertEqual(day, self.holidays.feasts[label])
	
	def testContains(self):
		h = set(self.expected.values())
		for day in h:
			self.assert_(day in self.holidays)
	
	def testNotContains(self):
		h = set(self.expected.values())
		base = date(2007, 1, 1)
		for dday in range(365):
			day = base+datetime.timedelta(dday)
			if day not in h:
				self.assert_(day not in self.holidays)

	def testIteration(self):
		for d, label in self.holidays:
			self.assertEqual(d, self.expected[label])

	def testPertaining(self):
		self.assert_(not self.holidays.isPertaining(date(2007, 12, 31)))
		self.assert_(self.holidays.isPertaining(date(2007, 1, 1)))


class IteratorTest(unittest.TestCase):
	def testIterForMonth(self):
		self.assertEqual(len([d for d in due.iterDaysForMonth(2, 2008)]), 29)
		self.assertEqual(len([d for d in due.iterDaysForMonth(2, 2007)]), 28)
		self.assertEqual(len([d for d in due.iterDaysForMonth(6, 2007)]), 30)
		self.assertEqual(len([d for d in due.iterDaysForMonth(7, 2008)]), 31)
	
	def testGetStartOfWeek(self):
		self.assertEqual(due.getStartOfWeek(12, 10, 2008), 
			datetime.date(2008, 10, 6))
		self.assertEqual(due.getStartOfWeek(1, 10, 2008), 
			datetime.date(2008, 9, 29))
		self.assertEqual(due.getStartOfWeek(1, 1, 2008), 
			datetime.date(2007, 12, 31))
	
	def testDaysForWeek(self):
		self.assertEqual([d for d in due.iterDaysForWeek(3, 1, 2008)],
			[date(2007, 12, 31), date(2008, 1, 1), date(2008, 1, 2),
				date(2008, 1, 3), date(2008, 1, 4), date(2008, 1, 5),
				date(2008, 1, 6)])

	def testFilterWorkdays(self):
		self.assertEqual([d for d in due.filterWorkdays(
				due.iterDaysBetween(date(2007, 12, 21), date(2008, 1, 10)))], [
			date(2007, 12, 21), date(2007, 12, 24), date(2007, 12, 27), 
			date(2007, 12, 28), date(2007, 12, 31), date(2008, 1, 2), 
			date(2008, 1, 3), date(2008, 1, 4), date(2008, 1, 7), 
			date(2008, 1, 8), date(2008, 1, 9), date(2008, 1, 10)])
	
	def testHoursForMonth(self):
		for (month, year), expected in [((1, 2008), 176), ((2, 2008), 168),
				((3, 2008), 152), ((4, 2008), 176)]:
			self.assertEqual(due.getHoursForMonth(month, year), expected)

	def testHoursForWeek(self):
		for dmy, expected in [((1,1,2008), 32), ((7,1,2008), 40),
				((24,3,2008), 32), ((31,3,2008), 40)]:
			self.assertEqual(due.getHoursForWeek(*dmy), expected)


if __name__=="__main__":
	testhelpers.main(IteratorTest)
